#
# Copyright (C) 2015 Jerome Kelleher <jerome.kelleher@well.ox.ac.uk>
#
# This file is part of kingman.
#
# kingman is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kingman is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kingman.  If not, see <http://www.gnu.org/licenses/>.
#
"""
The command line interface for the kingman package.
"""
from __future__ import print_function
from __future__ import division

import argparse
import json
import sys

import kingman


def get_parser():
    """
    Returns an argparse based command line argument parser for
    the kingman program.
    """
    description = (
        "A simple command line interface for the Kingman simulator. "
        "Outputs a simulated coalescent history in the form of an "
        "oriented forest in JSON format. Time is measured in units"
        "of 4Ne."
    )
    parser = argparse.ArgumentParser(description=description)
    parser.add_argument("sample_size", type=int, help="Sample size")
    parser.add_argument(
        "-s", "--random-seed", type=int, help="Random seed",
        default=None)
    return parser


def run_simulation(sample_size, random_seed, output):
    """
    Runs the simulation for the specified sample_size and random
    seed, writing out oriented forest output to the specified file
    in JSON format.
    """
    parent, time = kingman.simulate(sample_size, random_seed)
    d = {"parent": parent[1:], "time": time[1:]}
    json.dump(d, output)
    # Write a trailing newline.
    print(file=output)


def main():
    parser = get_parser()
    args = parser.parse_args()
    if args.sample_size < 2:
        parser.error("Sample size must be >= 2")
    run_simulation(args.sample_size, args.random_seed, sys.stdout)



#
# Copyright (C) 2015 Jerome Kelleher <jerome.kelleher@well.ox.ac.uk>
#
# This file is part of kingman.
#
# kingman is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kingman is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kingman.  If not, see <http://www.gnu.org/licenses/>.
#
"""
An example Python package, illustrating current best-practises.
"""
from __future__ import print_function
from __future__ import division

__version__ = '1.0.0'

# Import the functions here that we export in our public interface
from kingman.simulator import simulate  # NOQA



#
# Copyright (C) 2015 Jerome Kelleher <jerome.kelleher@well.ox.ac.uk>
#
# This file is part of kingman.
#
# kingman is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kingman is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kingman.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Simulation code for the kingman module.
"""
from __future__ import print_function
from __future__ import division

import random


def simulate(sample_size, random_seed=None):
    """
    Simulates the Kingman coalescent for the specified sample size
    and random seed.

    Returns a tuple (parent, time) which describes the simulated
    history of the sample as an oriented forest. ``parent`` is a
    list of integers, in which the parent of node ``j`` is
    ``parent[j]``. ``time`` an array of floating point values
    in which the time at which node ``j`` was created is ``time[j]``. See
    `<http://jeromekelleher.github.io/ercs/#oriented-trees-and-forests>`_
    for further information on oriented forests.

    Time is measured in units of 4Ne generations,
    following Hudson's ms.

    :param sample_size: The sample size; must be >= 2.
    :param random_seed: The random seed for simulations
    :type sample_size: integer
    :type random_seed: integer or None
    :rtype: (list, list)
    """
    if sample_size < 2:
        raise ValueError("sample_size must be >= 2")
    random.seed(random_seed)
    time = [0 for j in range(2 * sample_size)]
    parent = [0 for j in range(2 * sample_size)]
    # Following Knuth, we make the zeroth element -1.
    time[0] = -1
    parent[0] = -1
    ancestors = list(range(1, sample_size + 1))
    t = 0
    next_node = sample_size + 1
    for n in range(sample_size, 1, -1):
        t += random.expovariate(n * (n - 1))
        for _ in range(2):
            child = random.choice(ancestors)
            parent[child] = next_node
            ancestors.remove(child)
        ancestors.append(next_node)
        time[next_node] = t
        next_node += 1
    return parent, time



#
# Copyright (C) 2015 Jerome Kelleher <jerome.kelleher@well.ox.ac.uk>
#
# This file is part of kingman.
#
# kingman is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# kingman is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kingman.  If not, see <http://www.gnu.org/licenses/>.
#
import re

# First, we try to use setuptools. If it's not available locally,
# we fall back on ez_setup.
try:
    from setuptools import setup
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup


# Following the recommendations of PEP 396 we parse the version number
# out of the module.
def parse_version(module_file):
    """
    Parses the version string from the specified file.

    This implementation is ugly, but there doesn't seem to be a good way
    to do this in general at the moment.
    """
    f = open(module_file)
    s = f.read()
    f.close()
    match = re.findall("__version__ = '([^']+)'", s)
    return match[0]

f = open("README.txt")
kingman_readme = f.read()
f.close()
kingman_version = parse_version("kingman/__init__.py")

setup(
    name="kingman",
    version=kingman_version,
    description="An example Python package to simulate Kingman's coalescent",
    long_description=kingman_readme,
    packages=["kingman"],
    author="Jerome Kelleher",
    author_email="jerome.kelleher@well.ox.ac.uk",
    url="http://pypi.python.org/pypi/kingman",
    entry_points={
        'console_scripts': [
            'kingman=kingman.cli:main',
        ]
    },
    install_requires=[],
    keywords=["Coalescent simulation", "Python packaging"],
    license="GNU GPLv3+",
    platforms=["POSIX"],
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.1",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "Development Status :: 4 - Beta",
        "Environment :: Other Environment",
        "Intended Audience :: Science/Research",
        (
            "License :: OSI Approved :: GNU General Public License "
            "v3 or later (GPLv3+)"
        ),
        "Operating System :: POSIX",
        "Topic :: Scientific/Engineering",
        "Topic :: Scientific/Engineering :: Bio-Informatics",
    ],
)
